#version 330 core
struct LightProperties
{
	vec4 position;
	vec4 attenuation;
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
};

struct MaterialProperties
{
	vec4 ambientColor;
	vec4 diffuseColor;
	vec4 specularColor;
	float specularExponent;
};

uniform	LightProperties light[2];
uniform MaterialProperties material;
uniform float farDist;
uniform mat3 normalMatrix;

uniform sampler2D colorTexture;
uniform sampler2D normalMap;
uniform sampler2D specularMap;
uniform samplerCubeShadow shadowTexture[2];

in vec2 v_texCoord;
in vec3 v_tsLight1;
in vec3 v_tsLight2;
in vec3 v_tsEye;
in vec4 v_position;
in vec3 v_normal;

out vec4 out_color;

void main(void)
{ 
		vec4 texel = texture2D(colorTexture, v_texCoord);
		vec3 vertToEye = normalize(v_tsEye);
		vec4 color = vec4(0.0);

		vec4 specularColorT = texture2D(specularMap, v_texCoord);

		//vec4 color = vec4(0.0,0.0,0.0,1.0);
        // Convert the color information to a normal.
        vec3 normalDX = normalize( texture2D(normalMap, v_texCoord).xyz * 2.0 - 1.0 );
        // DirectX: Origin of uv is upper left. OpenGL: Origin of st is lower left. Normal vector points "towards" us in both cases.
        // As normal maps usually are created for DirectX, we have to convert from DirectX to OpenGL tangent space.
        vec3 normal;

        normal.x = dot(vec3(1.0, 0.0, 0.0), normalDX);
        normal.y = dot(vec3(0.0, -1.0, 0.0), normalDX);
        normal.z = dot(vec3(0.0, 0.0, 1.0), normalDX);

		for(int i=0; i < 2; i++)
		{	
			vec3 lightDir;
			if (i==0)
				lightDir = v_tsLight1;
			if (i==1)
				lightDir = v_tsLight2;
			float lightDist = length(lightDir);
			lightDir = normalize(lightDir);
			vec3 refl = normalize(reflect(lightDir, normal));
			float diffuseFactor = max(dot(normal, -normalize(lightDir)),0);
			float specularFactor = dot(vertToEye, refl);
			specularFactor = max(pow(specularFactor, material.specularExponent),0);

			float totalAttenuation = light[i].attenuation.x + light[i].attenuation.y * lightDist + light[i].attenuation.z * lightDist * lightDist;
			vec4 ambientColor = light[i].ambientColor*material.ambientColor / totalAttenuation;
			vec4 diffuseColor = diffuseFactor*light[i].diffuseColor*material.diffuseColor / totalAttenuation;
			vec4 specularColor = 8 * specularColorT * specularFactor*light[i].specularColor*material.specularColor / totalAttenuation;

			
			//Separate direction in worldspace for shadow maps
			vec3 shadowLightDir = v_position.xyz - light[i].position.xyz;
			lightDist = length(shadowLightDir);
			shadowLightDir = normalize(shadowLightDir);

			//Test to make sure this isn't the problem
			float diffuseFactorW = max(dot(normalize(v_normal), -shadowLightDir),0);
			float bias = 0.000005*tan(acos(diffuseFactorW));

			if(abs(shadowLightDir.x) > abs(shadowLightDir.y) && abs(shadowLightDir.x) > abs(shadowLightDir.z)){
				shadowLightDir.y = -shadowLightDir.y;
				shadowLightDir.z = -shadowLightDir.z;
			} else if(abs(shadowLightDir.z) > abs(shadowLightDir.y)) {
				shadowLightDir.x = -shadowLightDir.x;
				shadowLightDir.y = -shadowLightDir.y;
			}
			float sh = texture(shadowTexture[i], vec4(shadowLightDir,(lightDist / farDist)-bias));
			diffuseColor = diffuseColor * sh;
			specularColor = specularColor * sh;

			color = color + ambientColor + diffuseColor + specularColor;
		}
		out_color =  texel * color;

}